/*******************************************************************************
 * Copyright (c) 2003, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 * Port to the D programming language:
 *      John Reimer <terminal.node@gmail.com>
 *******************************************************************************/
module dwt.browser.HelperAppLauncherDialog;

import dwt.dwthelper.utils;

import Utf = tango.text.convert.Utf;

import dwt.DWT;

import XPCOM = dwt.internal.mozilla.XPCOM;

import dwt.internal.mozilla.Common;
import dwt.internal.mozilla.nsEmbedString;
import dwt.internal.mozilla.nsID;
import dwt.internal.mozilla.nsIHelperAppLauncher;
import dwt.internal.mozilla.nsIHelperAppLauncherDialog;
import dwt.internal.mozilla.nsIHelperAppLauncher_1_8;
import dwt.internal.mozilla.nsIHelperAppLauncher_1_9;
import dwt.internal.mozilla.nsISupports;
import dwt.internal.mozilla.nsILocalFile;
import dwt.internal.mozilla.nsStringAPI;

import dwt.browser.Mozilla;

import dwt.widgets.FileDialog;
import dwt.widgets.Shell;

class HelperAppLauncherDialog : nsIHelperAppLauncherDialog {
    int refCount = 0;

this() {
}

extern(System)
nsrefcnt AddRef () {
    refCount++;
    return refCount;
}

extern(System)
nsresult QueryInterface (nsID* riid, void** ppvObject) {
    if (riid is null || ppvObject is null) return XPCOM.NS_ERROR_NO_INTERFACE;
    
    if (*riid == nsISupports.IID) {
        *ppvObject = cast(void*)cast(nsISupports)this;
        AddRef ();
        return XPCOM.NS_OK;
    }
    if (*riid == nsIHelperAppLauncherDialog.IID) {
        *ppvObject = cast(void*)cast(nsIHelperAppLauncherDialog)this;
        AddRef ();
        return XPCOM.NS_OK;
    }
    
    *ppvObject = null;
    return XPCOM.NS_ERROR_NO_INTERFACE;
}

extern(System)
nsrefcnt Release () {
    refCount--;
    /*
    * Note.  This instance lives as long as the download it is binded to.
    * Its reference count is expected to go down to 0 when the download
    * has completed or when it has been cancelled. E.g. when the user
    * cancels the File Dialog, cancels or closes the Download Dialog
    * and when the Download Dialog goes away after the download is completed.
    */
    //if (refCount is 0) disposeCOMInterfaces ();
    return refCount;
}

/* nsIHelperAppLauncherDialog */

extern(System)
nsresult Show(nsIHelperAppLauncher aLauncher, nsISupports aContext, PRUint32 aReason) {
    /*
    * The interface for nsIHelperAppLauncher changed in GRE versions 1.8 and 1.9.  Query for
    * each of these interfaces in turn until one is found.
    */
    // FIXME: Not sure if this will work because of different versions of 
    // nsIHelperAppLauncher.  Remember to check. -JJR
    nsIHelperAppLauncher_1_9 helperAppLauncher1;
    int rc = aLauncher.QueryInterface (&nsIHelperAppLauncher_1_9.IID, cast(void**)&helperAppLauncher1);
    if (rc is 0) {
       // nsIHelperAppLauncher_1_9 helperAppLauncher = new nsIHelperAppLauncher_1_9 (aLauncher);
        rc = helperAppLauncher1.SaveToDisk (null, 0);
        helperAppLauncher1.Release ();
        return rc;
    }

    //result[0] = 0;
    nsIHelperAppLauncher_1_8 helperAppLauncher2;
    rc = aLauncher.QueryInterface (&nsIHelperAppLauncher_1_8.IID, cast(void**)&helperAppLauncher2);
    if (rc is 0) {
        //nsIHelperAppLauncher_1_8 helperAppLauncher = new nsIHelperAppLauncher_1_8 (aLauncher);
        rc = helperAppLauncher2.SaveToDisk (null, 0);
        helperAppLauncher2.Release ();
        return rc;
    }

    //nsIHelperAppLauncher helperAppLauncher = new nsIHelperAppLauncher (aLauncher);  /* < 1.8 */
    return aLauncher.SaveToDisk (null, 0);
}

extern(System)
nsresult PromptForSaveToFile (nsIHelperAppLauncher aLauncher, nsISupports aWindowContext, PRUnichar* aDefaultFile, PRUnichar* aSuggestedFileExtension, nsILocalFile* _retval) {
    bool hasLauncher = false;

    /*
    * The interface for nsIHelperAppLauncherDialog changed as of mozilla 1.5 when an
    * extra argument was added to the PromptForSaveToFile method (this resulted in all
    * subsequent arguments shifting right).  The workaround is to provide an XPCOMObject 
    * that fits the newer API, and to use the first argument's type to infer whether
    * the old or new nsIHelperAppLauncherDialog interface is being used (and by extension
    * the ordering of the arguments).  In mozilla >= 1.5 the first argument is an
    * nsIHelperAppLauncher. 
    */
    /*
     * The interface for nsIHelperAppLauncher changed as of mozilla 1.8, so the first
     * argument must be queried for both the old and new nsIHelperAppLauncher interfaces. 
     */
    bool using_1_8 = false, using_1_9 = false;
    // technically this should have an AddRef? -JJR
    //nsISupports support = cast(nsISupports)arg0; 
    //int /*long*/[] result = new int /*long*/[1];
    if (aLauncher is null)
        assert(0);
        
    // TODO: D Bindings are locked to the XPCOM interface version provided in
    // dwt/internal/mozilla package; therefore the following version checks 
    // are redundant for now. In this case, 1.8 and 1.9
    // will be bypassed.  To get 1.8 and 1.9 working for D, we will 
    // have to customize this section a bit or simply remove the references
    // until the default aligns with these versions in later swt updates. 
    // For now this should work as is.

    nsIHelperAppLauncher_1_8 helperAppLauncher1;

    int rc = aLauncher.QueryInterface (&nsIHelperAppLauncher_1_8.IID, cast(void**)&helperAppLauncher1);
    if (rc is 0) {
        using_1_8 = true;
        hasLauncher = true;
        helperAppLauncher1.Release ();
    } else {
        nsIHelperAppLauncher_1_9 helperAppLauncher2;
        rc = aLauncher.QueryInterface (&nsIHelperAppLauncher_1_9.IID, cast(void**)&helperAppLauncher2);
        if (rc is 0) {
            using_1_9 = true;
            hasLauncher = true;
            helperAppLauncher2.Release;
        } else {
            nsIHelperAppLauncher helperAppLauncher;
            rc = aLauncher.QueryInterface (&nsIHelperAppLauncher.IID, cast(void**)&helperAppLauncher);
            if (rc is 0) {
                hasLauncher = true;
                helperAppLauncher.Release;
            }
        }
    }

/+
    // In D port, Mozilla version is determined by the bindings used
    if (hasLauncher) {  /* >= 1.5 */
        aDefaultFile = arg2;
        aSuggestedFileExtension = arg3;
        _retval = arg4;
    } else {            /* 1.4 */  
    // This call conversion probablywon't work for non-Java
    // and shouldn't get called; fix it later. -JJR
        aDefaultFile = arg1;
        aSuggestedFileExtension = arg2;
        _retval = arg3;
    }
+/
    //int span = XPCOM.strlen_PRUnichar (aDefaultFile);
    // XPCOM.memmove (dest, aDefaultFile, length * 2);
    String defaultFile = Utf.toString(fromString16z(aDefaultFile));

    //span = XPCOM.strlen_PRUnichar (aSuggestedFileExtension);
    //dest = new char[length];
    //XPCOM.memmove (dest, aSuggestedFileExtension, length * 2);
    String suggestedFileExtension =  Utf.toString(fromString16z(aSuggestedFileExtension));

    Shell shell = new Shell ();
    FileDialog fileDialog = new FileDialog (shell, DWT.SAVE);
    fileDialog.setFileName (defaultFile);
    String[] tmp;
    tmp ~= suggestedFileExtension; 
    fileDialog.setFilterExtensions (tmp);
    String name = fileDialog.open ();
    shell.close ();
    if (name is null) {
        if (hasLauncher) {
            // not sure if these produce correct interface alignment. Check. -JJR
            if (using_1_8 || using_1_9) {
                //rc = aLauncher.Cancel (XPCOM.NS_BINDING_ABORTED);
                implMissing(__FILE__,__LINE__);
            } else {
                //nsIHelperAppLauncher launcher = new nsIHelperAppLauncher (arg0);
                rc = aLauncher.Cancel ();
            }
            if (rc !is XPCOM.NS_OK) Mozilla.error (rc); 
            return XPCOM.NS_OK;
        }
        return XPCOM.NS_ERROR_FAILURE;
    }
    scope auto path = new nsEmbedString (name.toString16());
    nsILocalFile localFile;
    rc = XPCOM.NS_NewLocalFile (cast(nsAString*)path, 1, &localFile);
    //path.dispose ();
    if (rc !is XPCOM.NS_OK) Mozilla.error (rc);
    if (localFile is null) Mozilla.error (XPCOM.NS_ERROR_NULL_POINTER);
    /* Our own nsIDownload has been registered during the Browser initialization. It will be invoked by Mozilla. */
    *_retval = localFile; 
    //XPCOM.memmove (_retval, result, C.PTR_SIZEOF);  
    return XPCOM.NS_OK;
}
}
